colors = {
    'pink': [.7, .7, .9],
    'purple': [.9, .7, .7],
    'cyan': [.7, .75, .5],
    'green': [.5, .75, .7],
    'yellow': [.5, .7, .75],
    'brown': [.5, .7, .7],
    'blue': [.75, .7, .5],
    'offwhite': [.8, .9, .9],
    'orange': [.5, .65, .9],
    'grey': [.7, .7, .7],
    }
def get_vertex_color_array():
    # Returns a per vertex RGB color
    # Assignment is random
    color_array = np.array(colors.values())
    vert_parts = pkl.load(open('data/partnames_per_vertex.pkl'))

    # vertex colors
    part_names = set(vert_parts)
    part_to_num = dict(zip(part_names, np.arange(len(part_names))))
    index = randint(len(colors), size=len(part_names))
    vert_ind = [part_to_num[vv] for vv in vert_parts] 
    vc = color_array[index[vert_ind]]
    return vc


if __name__ == '__main__':
    from cv2 import imread
    import cPickle as pkl
    import numpy as np
    from numpy.random import randint
    from opendr.renderer import ColoredRenderer
    from opendr.camera import ProjectPoints
    
    # vertices of mesh to render
    v = np.load('data/rigidv1.npy')
    f = np.load('data/rigidf1.npy')

    # Parameters that define the camera matrix
    # rt: rotation of camera in axis angle representation
    # t: transation
    # f: focal length
    # c: principal point of the camera
    # k: skew of pixels (zero)    
    
    parms = {'rt': np.array([1.2, 0.69, 2.53]),
             't': np.array([0.05, -.2, 2.17]),
             'f': [500, 500],
             'c': [250, 250],
             'k': np.zeros(8)}
    cam = ProjectPoints(**parms)

    # Define the renderer -- this is a color renderer which means 
    # there is no texture
    # Colors are defined per vertex and there is no lighting involved  
    vc = get_vertex_color_array()
    renderer = ColoredRenderer(v=v, 
                            f=f,
                            vc=vc,
                            camera=cam,
                            frustum={'near': .1, 'far': 10,
                                     'height': 500, 'width': 500},
                            bgcolor=np.array(np.zeros(3)),
                            overdraw=False)
    
    # Produce the image and save it
    image = renderer.r*255
    import cv2
    imfile = 'colored_mesh_image.png'
    print('Saving {}'.format(imfile))
    cv2.imwrite(imfile, image)
    print("Success!")
 

    


 
