import chumpy as ch
from .visualize_2pointclouds import visualize_2pointclouds
from .point2point import point2point_squared
import numpy as np


def icp_smpl_gradient(X, sm, faces0, fig, n_iters=30, objectives=['data']):
    '''
    Inputs:
    ------------
    X
        matrix of target coordinates.
    sm
        SMPL model parameterized by pose, betas and trans
    faces0
        matrix with the triangles corresponding to each vertices. Just
        for visualization
    fig
        mayavi figure
    n_iters
        number of iterations to perform in the icp algorithm
    objectives
        list of objectives to use in the minimization

    Outputs
    ------------
    d
        the residual out of the last procrustes iteration
    Z
        the matrix of transformed Y-values

    tform
        a dict specifying the rotation, translation and scaling that
        maps X --> Y

    '''
    muX = X.mean(0)
    # inverse of covariance matrix robustly estimated with sklearn.covariance.GraphLasso
    covariance_inv = np.diag(np.diag(np.load('data/precpose.npy')))
    # cholesky decomposition of covariance_inv with numpy.linalg.cholesky
    covariance_inv_chol = np.diag(np.diag(np.load('data/cholesky_precpose.npy')))
    mean = np.load('data/meanpose.npy')

    # initialize by matching the position of the centroids
    Xch = ch.asarray(X)
    #sm.pose[9] = 1.57
    #sm.pose[40] = -1.57
    #sm.pose[43] = 1.57
    sm.trans[:] = muX - sm.r.mean(0)  # initialize translation
    tm0, tm1 = visualize_2pointclouds(sm.r, sm.f, X, faces0, fig=fig)
    raw_input('set the camera viewpoint and press enter')

    # initialize transformed pointcloud
    # but this is a functional representation!
    def callback(_):
        tm0.mlab_source.set(points=sm.r)

    all_obj  = {'sum_on_data': point2point_squared(org_pts=sm, dst_pts=Xch)/Xch.shape[0],
	        'sum_on_model': point2point_squared(org_pts=Xch, dst_pts=sm)/sm.shape[0],
                'pose_prior': 1e-5*sm.pose,
		'shape_prior': 1e-5*sm.betas,
                'pose_prior_data': 1e-6*((sm.pose - mean).dot(covariance_inv).dot((sm.pose - mean).T)),
                'pose_prior_data_chol': 1e-6*(sm.pose - mean).dot(covariance_inv_chol)}

    ch.minimize({k:all_obj[k] for k in objectives}, x0=[sm.pose, sm.betas, sm.trans],
                options={'maxiter': 100, 'disp': 1, 'delta_0': .1, 'e_3': .01},
                method='dogleg',  # 'Newton-CG', 'BFGS', 'Nelder-Mead'...
                callback=callback)

    return(all_obj['sum_on_data'].sum().r, sm.r, {'translation': sm.trans.r,
		    'pose': sm.pose.r, 'shape': sm.betas})
