from mayavi.mlab import triangular_mesh, clf, figure, draw
import numpy as np
from os.path import join
import matplotlib.pyplot as plt

def visualize_2pointclouds(v0, f0, v1, f1, fig, representation=('surface', 'wireframe')):
    """
    Visualize a set of pointclouds point clouds

    Inputs:
    ------------
    v0, v1
        M_ix3 matrices of vertex positions

    f0, v1
        O_ix3 matrices containing indices to vertices in v[i] for each O_i triangles
    fig
        mayavi figure
    representation
        representation of each of the meshes

    """
    clf()
    # create triangulated meshes
    tm0 = triangular_mesh(v0.T[0], v0.T[1], v0.T[2], f0, color=(1., 0., 0.),
                          figure=fig, representation=representation[0])
    tm1 = triangular_mesh(v1.T[0], v1.T[1], v1.T[2], f1, color=(0., 0., 1.),
                          figure=fig, representation=representation[1])
    fig.scene.reset_zoom()
    return(tm0, tm1)


if __name__ == '__main__':
    data_path = '/tmp/06_BodyModels_bak/data/'
    fig = figure(size=(800,800))
    verts0 = np.load(join(data_path, 'rigidv0.npy'))
    faces0 = np.load(join(data_path,'rigidf0.npy'))
    verts1 = np.load(join(data_path, 'rigidv1.npy'))
    faces1 = np.load(join(data_path,'rigidf1.npy'))

    tm0, tm1 = visualize_pointclouds(verts1, faces1, verts0, faces0, fig=fig)
    verts1_copy = verts1.copy()
    for i in range(1000):
        verts1_copy[0] += 1.
        tm0.mlab_source.set(points=verts1_copy)
        print(i)
    raw_input('press enter')
