from chumpy import Ch
import chumpy as ch
from scipy.spatial import cKDTree

class point2point_squared(Ch):
    dterms = 'org_pts', 'dst_pts'

    # this represents the function evaluation
    def compute_r(self):
        # return the computed se
        return(self.se.r)

    # this the derivative computation
    # it actually exploits the chumpy autodiff
    # but we need to change the correspondences in each step
    def compute_dr_wrt(self, wrt):
        if wrt is self.org_pts:
            return(self.se.dr_wrt(self.org_pts))
        elif wrt is self.dst_pts:
            return(self.se.dr_wrt(self.dst_pts))
        else:
            return

    # this part is called automatically when variables change
    # to recompute the tree and recompute the correspondences
    def on_changed(self, which):
        # when the pts change, recompute the tree if needed
        # and recomputed the nearest pts # always

        # Recompute the tree for the reference pts when it changes
        if 'org_pts' in which:
            self.tree = cKDTree(self.org_pts.r.reshape((-1, 3)))

        # Recompute the correspondences
        self.dist, self.nearest_idx = self.tree.query(self.dst_pts)
        # compute the squared distances functional expression
        self.se = ch.sum((self.dst_pts - self.org_pts[self.nearest_idx])**2,
                          axis=1)


if __name__ == '__main__':
    import chumpy as ch
    import numpy as np
    org = ch.random.randn(100, 3)
    offset = ch.random.randn(1, 3)
    dst = org.r + offset
    p2p = point2point_squared(org_pts=org, dst_pts=dst)
    ch.minimize(p2p, x0=[offset])
    assert(np.linalg.norm(offset) < 1e-5)
    import ipdb; ipdb.set_trace()
